
function [Model cfg] = est_fitMVAR(varargin)
% Fit (adaptive) multivariate autoregressive model to EEG data. 
% See [1] for details on VAR model fitting and implementations.
%
% ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% Input                                  Information                                                                                                                                                                                                                                
% ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% ...| ALLEEG:                           EEGLAB dataset                                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: any evaluable Matlab expression.                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
% [+] Algorithm:                         Select a model fitting algorithm                                                                                                                                                                                                           
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        BSBL BO                                                                                                                                                                                                                                    
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: BSBL BO                                                                                                                                                                                                                       
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          BSBL Algorithm by Zhilin Zhang                                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: BSBL_BO()                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        BSBL L1                                                                                                                                                                                                                                    
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: BSBL L1                                                                                                                                                                                                                       
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          BSBL Algorithm by Zhilin Zhang                                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: BSBL_L1_noise()                                                                                                                                                                                                            
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        T-MSBL                                                                                                                                                                                                                                     
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: T-MSBL                                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          n/a                                                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: TMSBL()                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        ARfit                                                                                                                                                                                                                                      
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm:  ARfit                                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          Stepwise least squares estimation of the parameters                                                                                                                                                                                      
%                                          [A1, ... Ap] and noise covariance matrix C of a                                                                                                                                                                                          
%                                          multivariate autoregressive (VAR) model of order p:                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          v(t) = mu + A1*v(t-1) +...+ Ap*v(t-p) + noise(C)                                                                                                                                                                                         
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          This function is a wrapper for arfit() from the                                                                                                                                                                                          
%                                          ARFIT toolbox [1-3]                                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          [1] A. Neumaier and T. Schneider. ACM TOMS, 27(1):27-57, 2001.                                                                                                                                                                           
%                                          [2] T. Schneider and A. Neumaier. ACM TOMS, 27(1):58-65, 2001.                                                                                                                                                                           
%                                          [3] http://www.gps.caltech.edu/~tapio/arfit/                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: arfit()                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        Group Lasso (DAL/SCSA)                                                                                                                                                                                                                     
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: Group Lasso (DAL/SCSA)                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This method infers a fully connected multivariate                                                                                                                                                                                        
%                                          This program infers a sparsely connected multi-                                                                                                                                                                                          
%                                          variate AR (VAR) model [1] assuming that sources                                                                                                                                                                                         
%                                          are independent and super Gaussian (hyperbolic                                                                                                                                                                                           
%                                          secant likelihood). An option is also provided to                                                                                                                                                                                        
%                                          use a gaussian likelihood (squared loss function)                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          VAR[p] coefficients inferred using Group Lasso                                                                                                                                                                                           
%                                          (L1,2 regularization) via the Sparsely Connected                                                                                                                                                                                         
%                                          Sources Analysis (SCSA) and Dual Augmented                                                                                                                                                                                               
%                                          Lagrangian (DAL) methods [1][2].                                                                                                                                                                                                         
%                                          The p VAR coefficients describing interactions                                                                                                                                                                                           
%                                          between two processes at time lags [t-1...t-p]                                                                                                                                                                                           
%                                          are grouped together using an L2 norm which                                                                                                                                                                                              
%                                          penalizes large coefficients. An L1 penalty is                                                                                                                                                                                           
%                                          then applied to the coefficient groups. This                                                                                                                                                                                             
%                                          jointly prunes entire groups of coefficients                                                                                                                                                                                             
%                                          by setting the entire group to zero. The result                                                                                                                                                                                          
%                                          is a connectivity graph with sparse structure                                                                                                                                                                                            
%                                          (most processes are strictly non-interacting)                                                                                                                                                                                            
%                                          while regularizing (smoothing) the coefficient                                                                                                                                                                                           
%                                          sequences describing surviving non-zero                                                                                                                                                                                                  
%                                          interactions.                                                                                                                                                                                                                            
%                                          These constraints allow us to uniquely solve                                                                                                                                                                                             
%                                          highly under-determined systems (e.g. many                                                                                                                                                                                               
%                                          more parameters than observations).                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          If Y = D(y,p) is a p-lag delay embedding of multi-                                                                                                                                                                                       
%                                          variate data vector y(t), X is a sparse block-                                                                                                                                                                                           
%                                          diagonal matrix of lagged copies of the delay                                                                                                                                                                                            
%                                          embedded data, and A is an augmented matrix of                                                                                                                                                                                           
%                                          VAR coefficients, then we may adopt the structural                                                                                                                                                                                       
%                                          model:                                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                          Y = XA + e, for (super)gaussian noise e                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                          We then seek to solve the optimization problem                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          [A_hat mu] = argmin_A{f(Y-XA) + L*sum(||A_i||_2)}                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          Where f(z) specifies the unregularized portion of                                                                                                                                                                                        
%                                          the cost function (e.g. mimimizing variance of                                                                                                                                                                                           
%                                          residuals, e) and is given by:                                                                                                                                                                                                           
%                                          f(z) = ||z||_2^2  for gaussian innovations, and                                                                                                                                                                                          
%                                          f(z) = -sum(log((1/pi)sech(z))) for supergaussian                                                                                                                                                                                        
%                                          (sech) innovations.                                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          where A_i contains the i^th group of AR parameters                                                                                                                                                                                       
%                                          e.g. the set of VAR weights {A(i,j)} describing auto-                                                                                                                                                                                    
%                                          regression (conditional linear dependence) of X_i                                                                                                                                                                                        
%                                          onto X_j. L is the regularization parameter.                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This algorithm is a good choice if you have few                                                                                                                                                                                          
%                                          data samples and many channels/sources and/or a                                                                                                                                                                                          
%                                          high model order, or if the innovations are assumed                                                                                                                                                                                      
%                                          nongaussian (as with ICA sources).                                                                                                                                                                                                       
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          Adapted from s_test_hsgl.m from DAL 1.05 [3]                                                                                                                                                                                             
%                                          Copyright(c) 2009-2011 Ryota Tomioka                                                                                                                                                                                                     
%                                                       2009      Stefan Haufe                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          Reference:                                                                                                                                                                                                                               
%                                                                                                                                                                                                                                                                                   
%                                          [1] S. Haufe, R. Tomioka, G. Nolte, K-R. Mueller, M. Kawanabe. IEEE Trans. Biomed. Eng. 57(8),                                                                                                                                           
%                                        pp. 1954-1963, 2010.                                                                                                                                                                                                                       
%                                          [2] R. Tomioka, T. Suzuki, and M. Sugiyama. JMLR, 2011.                                                                                                                                                                                  
%                                          [3] DAL Toolbox: http://www.ibis.t.u-tokyo.ac.jp/ryotat/dal/                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: dalhsgl()                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        Group Lasso (ADMM)                                                                                                                                                                                                                         
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: Group Lasso (ADMM)                                                                                                                                                                                                            
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This method infers a sparsely connected multi-                                                                                                                                                                                           
%                                          variate autoregressive (VAR) model under a                                                                                                                                                                                               
%                                          gaussian noise assumption.                                                                                                                                                                                                               
%                                                                                                                                                                                                                                                                                   
%                                          VAR[p] coefficients inferred using Group Lasso                                                                                                                                                                                           
%                                          (L1,2 regularization) via the Alternating                                                                                                                                                                                                
%                                          Direction Method of Multipliers [1].                                                                                                                                                                                                     
%                                          The p VAR coefficients describing interactions                                                                                                                                                                                           
%                                          between two processes at time lags [t-1...t-p]                                                                                                                                                                                           
%                                          are grouped together using an L2 norm which                                                                                                                                                                                              
%                                          penalizes large coefficients. An L1 penalty is                                                                                                                                                                                           
%                                          then applied to the coefficient groups. This                                                                                                                                                                                             
%                                          jointly prunes entire groups of coefficients                                                                                                                                                                                             
%                                          by setting the entire group to zero. The result                                                                                                                                                                                          
%                                          is a connectivity graph with sparse structure                                                                                                                                                                                            
%                                          (most processes are strictly non-interacting)                                                                                                                                                                                            
%                                          while regularizing (smoothing) the coefficient                                                                                                                                                                                           
%                                          sequences describing surviving non-zero                                                                                                                                                                                                  
%                                          interactions.                                                                                                                                                                                                                            
%                                          These constraints allow us to uniquely solve                                                                                                                                                                                             
%                                          highly under-determined systems (e.g. many                                                                                                                                                                                               
%                                          more parameters than observations).                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
%                                          If Y = D(y,p) is a p-lag delay embedding of multi-                                                                                                                                                                                       
%                                          variate data vector y(t), X is a sparse block-                                                                                                                                                                                           
%                                          diagonal matrix of lagged copies of the delay                                                                                                                                                                                            
%                                          embedded data, and A is an augmented matrix of                                                                                                                                                                                           
%                                          VAR coefficients, then we may adopt the structural                                                                                                                                                                                       
%                                          model:                                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                          Y = XA + e,  for gaussian noise e                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          We then seek to solve the optimization problem                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          A_hat = argmin_A{0.5||Y-XA||_2^2 + L*sum(||A_i||_2)}                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          where A_i contains the i^th group of AR parameters                                                                                                                                                                                       
%                                          e.g. the set of VAR weights {A(i,j)} describing auto-                                                                                                                                                                                    
%                                          regression (conditional linear dependence) of X_i                                                                                                                                                                                        
%                                          onto X_j. L is the regularization parameter.                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This algorithm is a good choice if you have few                                                                                                                                                                                          
%                                          data samples and many channels/sources and/or a                                                                                                                                                                                          
%                                          high model order.                                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          This implementation is based on Matlab ADMM                                                                                                                                                                                              
%                                          examples from Stephen Boyd's website [2]                                                                                                                                                                                                 
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          [1] Boyd, Parikh, Chu, Pelato and Eckstein et al. Foundations and Trends in Machine Learning                                                                                                                                             
%                                        3(1):1-122,2011                                                                                                                                                                                                                            
%                                          [2] http://www.stanford.edu/~boyd/papers/distr_opt_stat_learning_admm.html                                                                                                                                                               
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: admm_gl()                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        Ridge Regression                                                                                                                                                                                                                           
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: Ridge Regression                                                                                                                                                                                                              
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This method infers a fully connected multivariate                                                                                                                                                                                        
%                                          autoregressive (VAR) model using L2 regularization,                                                                                                                                                                                      
%                                          otherwise known as "ridge-regression", "Tikhonov                                                                                                                                                                                         
%                                          regularization" or "minimum-L2-norm" [2]                                                                                                                                                                                                 
%                                                                                                                                                                                                                                                                                   
%                                          VAR[p] coefficients inferred using ridge regression                                                                                                                                                                                      
%                                          (L2 regularization). We assume most VAR coefficients                                                                                                                                                                                     
%                                          (interactions) are small, and apply an L2 penalty                                                                                                                                                                                        
%                                          for large coefficients. A consequence is that VAR                                                                                                                                                                                        
%                                          coefficients are never exactly zero; thus statistical                                                                                                                                                                                    
%                                          thresholding is an important follow-up step.                                                                                                                                                                                             
%                                          This constraint allows us to solve under-determined                                                                                                                                                                                      
%                                          systems (e.g. more parameters than observations).                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          If Y = D(y,p) is a p-lag delay embedding of multi-                                                                                                                                                                                       
%                                          variate data vector y(t), X is a block toeplitz                                                                                                                                                                                          
%                                          matrix of lagged copies of the delay embedded data,                                                                                                                                                                                      
%                                          and A is an augmented matrix of VAR coefficients,                                                                                                                                                                                        
%                                          then we may adopt the structural model:                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                          Y = XA + e,  for gaussian noise e                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
%                                          We then seek to solve the optimization problem                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          A_hat = argmin_A{0.5||Y-XA||_2^2 + L*||A||_2}                                                                                                                                                                                            
%                                                                                                                                                                                                                                                                                   
%                                          where L is the regularization parameter.                                                                                                                                                                                                 
%                                          L can be determined automatically using Generalized                                                                                                                                                                                      
%                                          Cross Validation (GCV) [3].                                                                                                                                                                                                              
%                                                                                                                                                                                                                                                                                   
%                                          This algorithm is a good choice if you have few                                                                                                                                                                                          
%                                          data samples and a fairly large number of sources                                                                                                                                                                                        
%                                          and/or a high model order. Note that, unlike sparse                                                                                                                                                                                      
%                                          methods, ridge regression yields a fully connected                                                                                                                                                                                       
%                                          graph (all VAR coefficients are non-zero). Follow up                                                                                                                                                                                     
%                                          with statistical thresholding.                                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          Implemented by Tim Mullen.                                                                                                                                                                                                               
%                                          The ridge regression implementation (ridgeGCV.m) was                                                                                                                                                                                     
%                                          contributed by Alejandro Ojeda (SCCN/INC).                                                                                                                                                                                               
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: ridge_gcv()                                                                                                                                                                                                                
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        SCSA EM                                                                                                                                                                                                                                    
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: SCSA EM                                                                                                                                                                                                                       
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          This implements the Sparsely Connected                                                                                                                                                                                                   
%                                          Sources Analysis (SCSA) algorithm of                                                                                                                                                                                                     
%                                          Haufe et al, 2010 [1]. Optimization is                                                                                                                                                                                                   
%                                          performed using an Expectation-                                                                                                                                                                                                          
%                                          Maximization (EM) algorithm.                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          The code for SCSA was contributed by                                                                                                                                                                                                     
%                                          Stefan Haufe, Ph.D [2]                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          [1] Haufe, S., Tomioka, R., & Nolte, G. (2010). Modeling sparse                                                                                                                                                                          
%                                              connectivity between underlying brain sources for EEG/MEG.                                                                                                                                                                           
%                                              Biomedical Engineering, (c), 1?10.                                                                                                                                                                                                   
%                                          [2] http://doc.ml.tu-berlin.de/publications/                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          Dependencies: scsa_em()                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                        Vieira-Morf                                                                                                                                                                                                                                
%                                        ---------------------------------------                                                                                                                                                                                                    
%                                          Algorithm: Vieira-Morf                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                          Description:                                                                                                                                                                                                                             
%                                                                                                                                                                                                                                                                                   
%                                          Unconstrained maximum entropy VAR modeling via                                                                                                                                                                                           
%                                          the Vieira-Morf lattice filter algorithm [2].                                                                                                                                                                                            
%                                                                                                                                                                                                                                                                                   
%                                          This algorithm is a good choice when you have                                                                                                                                                                                            
%                                          substantially more data than parameters to fit.                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          Author Credits:                                                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
%                                          This implementation is based on the mvar()                                                                                                                                                                                               
%                                          function from Alois Schloegl's BIOSIG and TSA                                                                                                                                                                                            
%                                          toolboxes [3][4]                                                                                                                                                                                                                         
%                                                                                                                                                                                                                                                                                   
%                                          References and Code:                                                                                                                                                                                                                     
%                                                                                                                                                                                                                                                                                   
%                                          [1] A. Schloegl, Comparison of Multivariate Autoregressive Estimators. Signal processing,                                                                                                                                                
%                                        Elsevier B.V.                                                                                                                                                                                                                              
%                                          [2] S.L. Marple. Digital Spectral Analysis with Applications. Prentice Hall, 1987                                                                                                                                                        
%                                          [3] http://biosig.sourceforge.net/                                                                                                                                                                                                       
%                                          [4] http://hci.tugraz.at/schloegl/matlab/tsa/                                                                                                                                                                                            
%                                        Possible values: {'BSBL BO', 'BSBL L1', 'T-MSBL', 'ARfit', 'Group Lasso (DAL/SCSA)', 'Group Lasso (ADMM)', 'Ridge Regression', 'SCSA EM', 'Vieira-Morf'}                                                                                   
%                                        Default value  : 'Vieira-Morf'                                                                                                                                                                                                             
%                                        Input Data Type: string                                                                                                                                                                                                                    
% ....[+] BSBL BO:                                                                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| NormCols:                     Normalize columns of dictionary                                                                                                                                                                                                            
%                                        Possible values: {'none', 'norm', 'zscore'}                                                                                                                                                                                                
%                                        Default value  : 'zscore'                                                                                                                                                                                                                  
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ....[+] WarmStart:                     Warm start                                                                                                                                                                                                                                 
%                                        The previously estimated state will be used as a starting estimate for successive BSBL operations.                                                                                                                                         
%                                        Alternately, you may provide an non-empty initial state structure via the 'InitialState' argument.                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial BSBL state object                                                                                                                                                                                                                  
%                                        If empty, last state will be used.                                                                                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| GroupAutoConnections:         Group auto-connections                                                                                                                                                                                                                     
%                                        All auto-connections for all channels will be penalized jointly as a separate group. Note, this can                                                                                                                                        
%                                        slow down model estimation as the design matrix will not longer be diagonal block-toeplitz so we                                                                                                                                           
%                                        cannot (yet) exploit block-redundancy in the design matrix                                                                                                                                                                                 
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ....[+] BSBL_Options:                  Additional options for BSBL algorithm                                                                                                                                                                                                      
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrix:             The design matrix                                                                                                                                                                                                                          
%                                        This is the data matrix (ie X).                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| TargetVector:             The target vector                                                                                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| Blocks:                   Block (Group) size                                                                                                                                                                                                                         
%                                        Array containing the number of elements in each block (group) of coefficients. ie: blks = [5 5 10]                                                                                                                                         
%                                        is two blocks of size 5 followed by a block of size 10.                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrixBlockSize:    Design matrix structure                                                                                                                                                                                                                    
%                                        If empty, do not exploit structure for faster computation. If non-empty, the design matrix consists                                                                                                                                        
%                                        of identical blocks along the main diagonal. 'DesignMatrixBlockSize' is the [numrow numcol] size of                                                                                                                                        
%                                        each block.                                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaLearningRule:       Lambda learning rule                                                                                                                                                                                                                       
%                                        'LowNoise' :  Use the lambda learning rule for very LOW SNR cases (SNR<10dB). Uses                                                                                                                                                         
%                                        lambda=std(y)*1e-2 or user-input value as initial value.                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                        'HighNoise':  Use the lambda learning rule for medium noisy cases (SNR>10dB). Uses                                                                                                                                                         
%                                        lambda=std(y)*1e-2 or user-input value as initial value.                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                        'off':        Do not use the lambda learning rule. Uses lambda=1e-14 or user-input value as initial                                                                                                                                        
%                                        value                                                                                                                                                                                                                                      
%                                        Possible values: {'off', 'LowNoise', 'HighNoise'}                                                                                                                                                                                          
%                                        Default value  : 'LowNoise'                                                                                                                                                                                                                
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| GammaPruningThresh:       Gamma Pruning threshold                                                                                                                                                                                                                    
%                                        Threshold for prunning small hyperparameters gamma_i. Blocks of parameters are pruned when their                                                                                                                                           
%                                        'power' (gamma_i) is small.                                                                                                                                                                                                                
%                                        In noisy cases, you can set PRUNE_GAMMA = 1e-3 or 1e-4.                                                                                                                                                                                    
%                                        In strong noisy cases (e.g. SNR <= 6 dB), set PRUNE_GAMMA = 1e-2 for better performance.                                                                                                                                                   
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| InitialLambda:            Initial regularization value (lambda)                                                                                                                                                                                                      
%                                        If empty, the initial value is estimated from the data.                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| StoppingTolerance:        Stopping tolerance                                                                                                                                                                                                                         
%                                        The algorithm terminates when the L2 norm of the change in parameter estimates is small than this                                                                                                                                          
%                                        value.                                                                                                                                                                                                                                     
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1e-08                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LearnCorrelation:         Exploit correlation structure                                                                                                                                                                                                              
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(true)                                                                                                                                                                                                             
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| MaxIterations:            Maximum iterations                                                                                                                                                                                                                         
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 600                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| VerboseOutput:            Verbosity                                                                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial BSBL state object for warm start                                                                                                                                                                                                   
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
% ....[+] BSBL L1:                                                                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| NormCols:                     Normalize columns of dictionary                                                                                                                                                                                                            
%                                        Possible values: {'none', 'norm', 'zscore'}                                                                                                                                                                                                
%                                        Default value  : 'zscore'                                                                                                                                                                                                                  
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ....[+] WarmStart:                     Warm start                                                                                                                                                                                                                                 
%                                        The previously estimated state will be used as a starting estimate for successive BSBL operations.                                                                                                                                         
%                                        Alternately, you may provide an non-empty initial state structure via the 'InitialState' argument.                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial BSBL state object                                                                                                                                                                                                                  
%                                        If empty, defaults will be used.                                                                                                                                                                                                           
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| GroupAutoConnections:         Group auto-connections                                                                                                                                                                                                                     
%                                        All auto-connections for all channels will be penalized jointly as a separate group. Note, this can                                                                                                                                        
%                                        slow down model estimation as the design matrix will not longer be diagonal block-toeplitz so we                                                                                                                                           
%                                        cannot (yet) exploit block-redundancy in the design matrix                                                                                                                                                                                 
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ....[+] BSBL_Options:                  Additional options for BSBL algorithm                                                                                                                                                                                                      
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaLearningRule:       Lambda learning rule                                                                                                                                                                                                                       
%                                        'LowNoise' :  Use the lambda learning rule for very LOW SNR cases (SNR<10dB). Uses                                                                                                                                                         
%                                        lambda=std(y)*1e-2 or user-input value as initial value.                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                        'HighNoise':  Use the lambda learning rule for medium noisy cases (SNR>10dB). Uses                                                                                                                                                         
%                                        lambda=std(y)*1e-2 or user-input value as initial value.                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
%                                        'off':        Do not use the lambda learning rule. Uses lambda=1e-14 or user-input value as initial                                                                                                                                        
%                                        value                                                                                                                                                                                                                                      
%                                        Possible values: {'off', 'LowNoise', 'HighNoise'}                                                                                                                                                                                          
%                                        Default value  : 'LowNoise'                                                                                                                                                                                                                
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| GammaPruningThreshold:    Gamma Pruning threshold                                                                                                                                                                                                                    
%                                        Threshold for prunning small hyperparameters gamma_i. Blocks of parameters are pruned when their                                                                                                                                           
%                                        'power' (gamma_i) is small.                                                                                                                                                                                                                
%                                        In noisy cases, you can set PRUNE_GAMMA = 1e-3 or 1e-4.                                                                                                                                                                                    
%                                        In strong noisy cases (e.g. SNR <= 6 dB), set PRUNE_GAMMA = 1e-2 for better performance.                                                                                                                                                   
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| InitialLambda:            Initial regularization value (lambda)                                                                                                                                                                                                      
%                                        If empty, the initial value is estimated from the data.                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| StoppingTolerance:        Stopping tolerance                                                                                                                                                                                                                         
%                                        The algorithm terminates when the L2 norm of the change in parameter estimates is small than this                                                                                                                                          
%                                        value.                                                                                                                                                                                                                                     
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1e-08                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LearnCorrelation:         Exploit correlation structure                                                                                                                                                                                                              
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(true)                                                                                                                                                                                                             
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| MaxIterations:            Maximum iterations                                                                                                                                                                                                                         
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| VerboseOutput:            Verbosity                                                                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
% ....[+] T-MSBL:                                                                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| Verbosity:                    Verbose output                                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 0                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
% ....[+] ARfit:                                                                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         [chans x time x trials] data matrix                                                                                                                                                                                                        
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   Maximum model order                                                                                                                                                                                                                        
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| OrderSelector:                Model order selection criterion                                                                                                                                                                                                            
%                                        Possible values: {'sbc', 'fpe'}                                                                                                                                                                                                            
%                                        Default value  : 'sbc'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| NoConstantTerm:               Exclude constant term in model                                                                                                                                                                                                             
%                                        Setting this implicitly assumes the process mean is zero.                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(true)                                                                                                                                                                                                             
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
% ....[+] Group Lasso (DAL/SCSA):                                                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| InitialState:                 Initial VAR coefficient matrix                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: any evaluable Matlab expression.                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
% ....[+] DAL_Options:                   Options for DAL algorithm                                                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| RegularizationParam:      Regularization parameter                                                                                                                                                                                                                   
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| ShrinkDiagonal:           Apply sparsity constraint to diagonals (self-connections)                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(true)                                                                                                                                                                                                             
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| LossFunction:             Optimization loss function                                                                                                                                                                                                                 
%                                        Possible values: {'HyperbolicSecant', 'Squared'}                                                                                                                                                                                           
%                                        Default value  : 'HyperbolicSecant'                                                                                                                                                                                                        
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| Verbosity:                Verbosity Level. 0 = basic, 2 = detailed                                                                                                                                                                                                   
%                                        Possible values: {int32(0), int32(1), int32(2)}                                                                                                                                                                                            
%                                        Default value  : 0                                                                                                                                                                                                                         
%                                        Input Data Type: integer (int32)                                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
% ...........| DAL_NVP_Args:             Name-value pairs for DAL                                                                                                                                                                                                                   
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : '{}'                                                                                                                                                                                                                      
%                                        Input Data Type: any evaluable Matlab expression.                                                                                                                                                                                          
% ....[+] Group Lasso (ADMM):                                                                                                                                                                                                                                                       
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ....[+] WarmStart:                     Warm start                                                                                                                                                                                                                                 
%                                        The previously estimated state will be used as a starting estimate for successive operations.                                                                                                                                              
%                                        Alternately, you may provide an non-empty initial state structure via the 'InitialState' argument.                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial ADMM state                                                                                                                                                                                                                         
%                                        This is a structure with fields 'z_init' and 'u_init', which represent, respectively, the initial                                                                                                                                          
%                                        state vector and dual vector and are both of dimension [morder*(nchs^2) x 1]. If empty, the first                                                                                                                                          
%                                        state is initialized to zeros.                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| NormCols:                     Normalize columns of dictionary                                                                                                                                                                                                            
%                                        Possible values: {'none', 'norm', 'zscore'}                                                                                                                                                                                                
%                                        Default value  : 'none'                                                                                                                                                                                                                    
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| GroupAutoConnections:         Group auto-connections                                                                                                                                                                                                                     
%                                        All auto-connections for all channels will be penalized jointly as a separate group. Note, this can                                                                                                                                        
%                                        slow down model estimation as the design matrix will not longer be diagonal block-toeplitz so we                                                                                                                                           
%                                        cannot (yet) exploit block-redundancy in the design matrix                                                                                                                                                                                 
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| InitialState:                 DEPRECATED                                                                                                                                                                                                                                 
%                                        Initial VAR coefficient matrix                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: any evaluable Matlab expression.                                                                                                                                                                                          
%                                                                                                                                                                                                                                                                                   
% ....[+] GPU:                           Use GPU                                                                                                                                                                                                                                    
%                                        This uses the CUDA_ADMM package by Oleg Konigs.                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'off'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| Device:                   Which GPU device to use                                                                                                                                                                                                                    
%                                        Possible values: {'default'}                                                                                                                                                                                                               
%                                        Default value  : 'default'                                                                                                                                                                                                                 
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| Library:                  Which library to use                                                                                                                                                                                                                       
%                                        Possible values: {'CUDA_ADMM'}                                                                                                                                                                                                             
%                                        Default value  : 'CUDA_ADMM'                                                                                                                                                                                                               
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ....[+] ADMM_Options:                  Options for ADMM algorithm                                                                                                                                                                                                                 
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrix:             The design matrix                                                                                                                                                                                                                          
%                                        This is the data matrix (ie X).                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| TargetVector:             The target vector                                                                                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| Blocks:                   Block (Group) size                                                                                                                                                                                                                         
%                                        Array containing the number of elements in each block (group) of coefficients. ie: blks = [5 5 10]                                                                                                                                         
%                                        is two blocks of size 5 followed by a block of size 10.                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrixBlockSize:    Design matrix structure                                                                                                                                                                                                                    
%                                        If empty, do not exploit structure for faster computation. If non-empty, the design matrix consists                                                                                                                                        
%                                        of identical blocks along the main diagonal. 'DesignMatrixBlockSize' is the [numrow numcol] size of                                                                                                                                        
%                                        each block.                                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| ReguParamLambda:          Regularization parameter (lambda)                                                                                                                                                                                                          
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| AugLagrangParamRho:       Initial value of augmented Lagrangian parameter (rho)                                                                                                                                                                                      
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| OverRelaxationParamAlpha: Over-relaxation parameter (alpha)                                                                                                                                                                                                          
%                                        Typical values are between 1.5 and 1.8                                                                                                                                                                                                     
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1.7                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial state vector                                                                                                                                                                                                                       
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| InitialDual:              Initial value of u                                                                                                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| Verbosity:                Verbose output                                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| MaxIterations:            Maximum number of iterations                                                                                                                                                                                                               
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 1000                                                                                                                                                                                                                      
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| ComputeObjectiveValue:    Compute objective value function                                                                                                                                                                                                           
%                                        Slower processing. Useful mainly for diagnostics                                                                                                                                                                                           
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| RhoUpdate:                Update Rho                                                                                                                                                                                                                                 
%                                        Whether to update rho dynamically according to 3.4.1 in [1]. Note, this can sometimes cause r_norm,                                                                                                                                        
%                                        s_norm to "blow up"                                                                                                                                                                                                                        
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| RhoUpdateThreshold:       Rho update threshold                                                                                                                                                                                                                       
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| RhoUpdateIncr:            Rho update increment factor                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 2                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| RhoUpdateDecr:            Rho update decrement factor                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 2                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaUpdate:             Update Lambda                                                                                                                                                                                                                              
%                                        Whether to decrease lambda (regularization) dynamically if convergence is slow.                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaUpdateThreshold:    Update lambda if the change in r_norm is less than this                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 1e-05                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaUpdateCount:        Number of iterations before updating lambda                                                                                                                                                                                                
%                                        Update lambda if r_norm has not significantly changed after this many iterations.                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| LambdaUpdateFactor:       Update factor by which to divide lambda                                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| AbsoluteTolerance:        Primal/Dual absolute tolerance                                                                                                                                                                                                             
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 0.0001                                                                                                                                                                                                                    
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| RelativeTolerance:        Primal/Dual relative tolerance                                                                                                                                                                                                             
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 0.01                                                                                                                                                                                                                      
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ........[+] SolverMethod:              Method used to update parameters x                                                                                                                                                                                                         
%                                        If 'direct' then a cholesky factorization of A (data/design matrix) is pre-cached and used to                                                                                                                                              
%                                        update x directly. This is usually a good choice, unless A is very large.                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
%                                        If 'iterative', then x is updated using an iterative method (lsqr.m)                                                                                                                                                                       
%                                        Possible values: {'direct', 'iterative'}                                                                                                                                                                                                   
%                                        Default value  : 'direct'                                                                                                                                                                                                                  
%                                        Input Data Type: string                                                                                                                                                                                                                    
% ............[+] direct:                                                                                                                                                                                                                                                           
% ............[+] iterative:                                                                                                                                                                                                                                                        
%                                                                                                                                                                                                                                                                                   
% ...............| MaxIterations:        Number of iterations                                                                                                                                                                                                                       
%                                        If unspecified, a suitable number is automatically determined by lsqr()                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...............| Tolerance:            Stopping tolerance                                                                                                                                                                                                                         
%                                        Iteration terminates when residual norm is smaller than this value. Default value is 1e-6. However,                                                                                                                                        
%                                        this can be relaxed substantially as described in section 4.3 of [1].                                                                                                                                                                      
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 1e-06                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
% ....[+] Ridge Regression:                                                                                                                                                                                                                                                         
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts x ntr].                                                                                                                                                                                                       
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| NormCols:                     Normalize data                                                                                                                                                                                                                             
%                                        Columns of data matrix X and target vector Y are normalized using the chosen method.                                                                                                                                                       
%                                        Possible values: {'none', 'norm', 'zscore'}                                                                                                                                                                                                
%                                        Default value  : 'norm'                                                                                                                                                                                                                    
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ....[+] DecoupleVars:                  Decouple variables                                                                                                                                                                                                                         
%                                        Solve coefficients for each variable separately. This can improve computation speed when there are                                                                                                                                         
%                                        a large number of channels/variables                                                                                                                                                                                                       
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'on'                                                                                                                                                                                                                      
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ........[+] RunInParallel:             Solve blocks in parallel                                                                                                                                                                                                                   
%                                        Requires Parallel Computing Toolbox.                                                                                                                                                                                                       
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'off'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...............| ProfileName:          Profile name                                                                                                                                                                                                                               
%                                        Possible values: {'local'}                                                                                                                                                                                                                 
%                                        Default value  : 'local'                                                                                                                                                                                                                   
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...............| NumWorkers:           Number of workers                                                                                                                                                                                                                          
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 2                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ....[+] RegressionOptions:             Ridge regression options                                                                                                                                                                                                                   
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| TargetVector:             The target vector                                                                                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrix:             The design matrix                                                                                                                                                                                                                          
%                                        This is the data matrix (ie X).                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| PriorInvCov:              Prior inverse covariance (precision) matrix for params                                                                                                                                                                                     
%                                        Can be an [nc x nc] matrix, where nc is the number of columns of A. Can also be a scalar, P,                                                                                                                                               
%                                        specifying the prior inverse variance (precision) of each parameter (diagonal covariance matrix).                                                                                                                                          
%                                        If empty, identity covariance matrix assumed. A sparse matrix is advised if precision matrix is not                                                                                                                                        
%                                        dense.                                                                                                                                                                                                                                     
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| DesignMatrixBlockSize:    Design matrix structure                                                                                                                                                                                                                    
%                                        Can be a tuple [numrows numcols], in which case A consists of identical blocks of this size, along                                                                                                                                         
%                                        the main diagonal                                                                                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ........[+] LambdaSelectionMode:       Selection mode for lambda                                                                                                                                                                                                                  
%                                        Automatic (GCV grid search) or Manual (must provide lambda)                                                                                                                                                                                
%                                        Possible values: {'manual', 'grid_gcv'}                                                                                                                                                                                                    
%                                        Default value  : 'grid_gcv'                                                                                                                                                                                                                
%                                        Input Data Type: string                                                                                                                                                                                                                    
% ............[+] manual:                                                                                                                                                                                                                                                           
%                                                                                                                                                                                                                                                                                   
% ...............| RegularizationParam:  Regularization parameter (lambda)                                                                                                                                                                                                          
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
% ............[+] grid_gcv:                                                                                                                                                                                                                                                         
%                                                                                                                                                                                                                                                                                   
% ...............| GridSize:             Grid size for regularization param search                                                                                                                                                                                                  
%                                        This is used to automatically select the regularization parameter which minimizes the Generalized                                                                                                                                          
%                                        Cross-Validation (GCV) criteria.                                                                                                                                                                                                           
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 100                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...............| PlotGCV:              Plot GCV curve                                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| svd_state:                SVD decomposition structure                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| Verbosity:                Verbose output                                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| Verbosity:                    Verbose output                                                                                                                                                                                                                             
%                                        Possible values: {int32(0), int32(1), int32(2)}                                                                                                                                                                                            
%                                        Default value  : 0                                                                                                                                                                                                                         
%                                        Input Data Type: integer (int32)                                                                                                                                                                                                           
% ....[+] SCSA EM:                                                                                                                                                                                                                                                                  
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data Matrix                                                                                                                                                                                                                                
%                                        Dimensions are [nchs x npnts].                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   VAR Model order                                                                                                                                                                                                                            
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ....[+] WarmStart:                     Warm start                                                                                                                                                                                                                                 
%                                        The previously estimated state will be used as a starting estimate for successive operations.                                                                                                                                              
%                                        Alternately, you may provide an non-empty initial state structure via the 'InitialState' argument.                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             Initial SCSA state                                                                                                                                                                                                                         
%                                        This is a structure with fields 'B' and 'H', which represent, respectively, the initial [M x M] ICA                                                                                                                                        
%                                        demixing matrix and the [M x M x morder] MVAR coefficient matrix.                                                                                                                                                                          
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ....[+] PCA:                           Reduce dimensionality with PCA                                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'on'                                                                                                                                                                                                                      
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...........| NumComps:                 Number of PCs to retain                                                                                                                                                                                                                    
%                                        If empty, retain all components. If dim < 1 this is considered the proportion of signal variance                                                                                                                                           
%                                        explained to retain                                                                                                                                                                                                                        
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| InitialState:             PCA filter structure                                                                                                                                                                                                                       
%                                        This contains fields 'pcaweights' and 'pcawinv' containing PCA filter weights and patterns                                                                                                                                                 
%                                        (loadings), respectively                                                                                                                                                                                                                   
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| DataIsCentered:           Set to true if data is zero mean                                                                                                                                                                                                           
%                                        Otherwise mean will be removed from each channel/trial                                                                                                                                                                                     
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ....[+] SCSA_Options:                  Options for ADMM algorithm                                                                                                                                                                                                                 
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...........| ReguParamLambda:          Regularization parameter (lambda)                                                                                                                                                                                                          
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 0.2                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| NumRestarts:              Number of EM restarts                                                                                                                                                                                                                      
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| MaxIterations:            Maximum EM iterations                                                                                                                                                                                                                      
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 5                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...........| Tolerance:                EM stopping tolerance                                                                                                                                                                                                                      
%                                        Possible values: [0 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 1e-06                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| VerbosityLevel:               Verbosity level. 0 = no output, 1 = text, 2 = graphical                                                                                                                                                                                    
%                                        Possible values: {int32(0), int32(1), int32(2)}                                                                                                                                                                                            
%                                        Default value  : 2                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
% ....[+] Vieira-Morf:                                                                                                                                                                                                                                                              
%                                                                                                                                                                                                                                                                                   
% .......| Data:                         Data matrix                                                                                                                                                                                                                                
%                                        Format must be [channels x time x trials]                                                                                                                                                                                                  
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| ModelOrder:                   Maximum model order                                                                                                                                                                                                                        
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| ModelOrder:                       VAR model order                                                                                                                                                                                                                            
%                                        This overrides lower-level ModelOrder options.                                                                                                                                                                                             
%                                        Possible values: [1 Inf]                                                                                                                                                                                                                   
%                                        Default value  : 10                                                                                                                                                                                                                        
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| WindowStartIndices:               Starting indices for windows                                                                                                                                                                                                               
%                                        This is a vector of sample points (start of windows) at which to estimate windowed VAR model                                                                                                                                               
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| WindowLength:                     Sliding window length (sec)                                                                                                                                                                                                                
%                                        Possible values: [2.22044604925031e-16 Inf]                                                                                                                                                                                                
%                                        Default value  : 0.5                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| WindowStepSize:                   Window step size (sec)                                                                                                                                                                                                                     
%                                        Possible values: [2.22044604925031e-16 Inf]                                                                                                                                                                                                
%                                        Default value  : 0.03                                                                                                                                                                                                                      
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| TaperFunction:                    Data tapering (windowing) function                                                                                                                                                                                                         
%                                        Each data segment (e.g. all data in a sliding window) will be multipled by the data taper to smooth                                                                                                                                        
%                                        endpoints towards zero.                                                                                                                                                                                                                    
%                                        Available windows are:                                                                                                                                                                                                                     
%                                        rectwin          - Rectangular window. This is equivalent to no taper.                                                                                                                                                                     
%                                        hamming          - Hamming window.                                                                                                                                                                                                         
%                                        hann          - Hann window.                                                                                                                                                                                                               
%                                        bartlett          - Bartlett window.                                                                                                                                                                                                       
%                                        barthannwin          - Modified Bartlett-Hanning window.                                                                                                                                                                                   
%                                        blackman          - Blackman window.                                                                                                                                                                                                       
%                                        blackmanharris- Minimum 4-term Blackman-Harris window.                                                                                                                                                                                     
%                                        bohmanwin          - Bohman window.                                                                                                                                                                                                        
%                                        chebwin          - Chebyshev window.                                                                                                                                                                                                       
%                                        flattopwin          - Flat Top window.                                                                                                                                                                                                     
%                                        gausswin          - Gaussian window.                                                                                                                                                                                                       
%                                        kaiser          - Kaiser window.                                                                                                                                                                                                           
%                                        nuttallwin          - Nuttall defined minimum 4-term Blackman-Harris window.                                                                                                                                                               
%                                        parzenwin          - Parzen (de la Valle-Poussin) window.                                                                                                                                                                                  
%                                        taylorwin          - Taylor window.                                                                                                                                                                                                        
%                                        tukeywin          - Tukey window.                                                                                                                                                                                                          
%                                        triang          - Triangular window.                                                                                                                                                                                                       
%                                        Possible values: {'rectwin', 'hamming', 'hann', 'bartlett', 'barthannwin', 'blackman', 'blackmanharris', 'bohmanwin', 'chebwin', 'flattopwin', 'gausswin', 'kaiser', 'nuttallwin', 'parzenwin', 'taylorwin', 'tukeywin', 'triang'}         
%                                        Default value  : 'rectwin'                                                                                                                                                                                                                 
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...| EpochTimeLimits:                  Sub-epoch time limits (sec)                                                                                                                                                                                                                
%                                        This is relative to event time (e.g. [-1 2]). Default is the full epoch time range                                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% ...| WindowSamplePercent:              Percent of windows to sample                                                                                                                                                                                                               
%                                        Possible values: [1 100]                                                                                                                                                                                                                   
%                                        Default value  : 100                                                                                                                                                                                                                       
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% [+] NormalizeData:                     Z-normalize data within windows                                                                                                                                                                                                            
%                                        Note this is not recommended for short windows                                                                                                                                                                                             
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| data:                                                                                                                                                                                                                                                                    
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'MANDATORY INPUT'                                                                                                                                                                                                         
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% .......| VerbosityLevel:               Verbosity level. 0 = no output, 1 = text, 2 = graphical                                                                                                                                                                                    
%                                        Possible values: {int32(0), int32(1), int32(2)}                                                                                                                                                                                            
%                                        Default value  : 0                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)                                                                                                                                                                                                      
%                                                                                                                                                                                                                                                                                   
% .......| Method:                       Normalize windows across time, ensemble, or both                                                                                                                                                                                           
%                                        Possible values: {'time', 'ensemble'}                                                                                                                                                                                                      
%                                        Default value  : {'time', 'ensemble'}                                                                                                                                                                                                      
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% [+] Detrend:                           Detrend or center each time window                                                                                                                                                                                                         
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : 'n/a'                                                                                                                                                                                                                     
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% .......| DetrendingMethod:             Detrend data within each window                                                                                                                                                                                                            
%                                        Linear: removes the least-squares fit of a straight line.                                                                                                                                                                                  
%                                        Constant: removes the mean from each trial (centering)                                                                                                                                                                                     
%                                        Possible values: {'linear', 'constant'}                                                                                                                                                                                                    
%                                        Default value  : 'constant'                                                                                                                                                                                                                
%                                        Input Data Type: string                                                                                                                                                                                                                    
%                                                                                                                                                                                                                                                                                   
% ...| Timer:                            Activate timer                                                                                                                                                                                                                             
%                                        Times are stored in EEG.CAT.Model.timeelapsed                                                                                                                                                                                              
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(false)                                                                                                                                                                                                            
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...| SetArgDirectMode:                 Set arg_direct mode to true                                                                                                                                                                                                                
%                                        Can improve speed when number of windows is large. Disable if calling this function repeatedly in a                                                                                                                                        
%                                        tight loop.                                                                                                                                                                                                                                
%                                        Possible values: 'Unrestricted'                                                                                                                                                                                                            
%                                        Default value  : logical(true)                                                                                                                                                                                                             
%                                        Input Data Type: boolean                                                                                                                                                                                                                   
%                                                                                                                                                                                                                                                                                   
% ...| VerbosityLevel:                   Verbosity level. 0 = no output, 1 = text, 2 = graphical                                                                                                                                                                                    
%                                        Possible values: {int32(0), int32(1), int32(2)}                                                                                                                                                                                            
%                                        Default value  : 2                                                                                                                                                                                                                         
%                                        Input Data Type: real number (double)   
%
% ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
% Output:
% ----------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
%
%   Model               MODEL structure with
%                       .Model          (numvars x coeffs) matrix of VAR coefficients
%                       .PE             (numvars x coeffs) prediction error (noise covariance) coefficients
%                       .algorithm      string denoting algorithm used for estimation
%                       .modelclass     string denoting model class (here, 'mvar')
%
%   cfg                 arg config structure
%
% See Also: pop_est_fitMVAR(), pop_pre_prepData()
%
% References:
%
% [1] Mullen T (2010) The Source Information Flow Toolbox (SIFT):
%   Theoretical Handbook and User Manual. Chapters 3,6.
%   Available at: http://www.sccn.ucsd.edu/wiki/Sift/
%
%
% Author: Tim Mullen 2010, SCCN/INC, UCSD.
% Email:  tim@sccn.ucsd.edu

% This function is part of the Source Information Flow Toolbox (SIFT)
%
% This program is free software; you can redistribute it and/or modify
% it under the terms of the GNU General Public License as published by
% the Free Software Foundation; either version 3 of the License, or
% (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

verb = arg_extract(varargin,{'verb','VerbosityLevel'},[],2);

haveSigProc = hlp_isToolboxInstalled('Signal Processing Toolbox');
if haveSigProc
    taperFcns = {'rectwin','hamming','hann','bartlett','barthannwin',   ...
                 'blackman','blackmanharris','bohmanwin','chebwin',     ...
                 'flattopwin','gausswin','kaiser','nuttallwin',         ...
                 'parzenwin','taylorwin','tukeywin','triang'};
else
    taperFcns = {'rectwin'};   
end

g = arg_define([0 1],varargin, ...
    arg_norep({'EEG','ALLEEG'},mandatory,[],'EEGLAB dataset','type','expression'), ...
    arg_subswitch({'algorithm','Algorithm'},hlp_microcache('algos',@hlp_getMVARalgorithms,'defaultNameOnly'), ...
        hlp_microcache('algos',@hlp_getMVARalgorithms), ...
        {'Select a model fitting algorithm.',hlp_microcache('sift_domain',@hlp_buildMVARHelpText)}, ...
        'cat','Modeling Parameters','suppress',{'ModelOrder','OrderSelector','InitialState'}), ...
    arg({'morder','ModelOrder','modelOrder'},10,[1 Inf],'VAR model order. This overrides lower-level ModelOrder options.','cat','Modeling Parameters'), ...
    arg_nogui({'winStartIdx','WindowStartIndices'},[],[],'Starting indices for windows. This is a vector of sample points (start of windows) at which to estimate windowed VAR model','cat','Modeling Parameters'), ...
    arg({'winlen','WindowLength'},0.5,[eps Inf],'Sliding window length (sec)','cat','Modeling Parameters'), ...
    arg({'winstep','WindowStepSize'},0.03,[eps Inf],'Window step size (sec)','cat','Modeling Parameters'), ...
    arg({'taperfcn','TaperFunction'},'rectwin',taperFcns,{'Data tapering (windowing) function', ...
                                                            sprintf(['\n' ...
                                                                     'Each data segment (e.g. all data in a sliding window) will be multipled by the data taper to smooth endpoints towards zero.', ...
                                                                     '\n' ...
                                                                     'Available windows are:\n' ...
                                                                          'rectwin \t - Rectangular window. This is equivalent to no taper.\n', ...
                                                                          'hamming \t - Hamming window.\n', ...
                                                                          'hann \t - Hann window.\n', ...
                                                                          'bartlett \t - Bartlett window.\n', ...
                                                                          'barthannwin \t - Modified Bartlett-Hanning window.\n', ...
                                                                          'blackman \t - Blackman window.\n', ...
                                                                          'blackmanharris- Minimum 4-term Blackman-Harris window.\n', ...
                                                                          'bohmanwin \t - Bohman window.\n', ...
                                                                          'chebwin \t - Chebyshev window.\n', ...
                                                                          'flattopwin \t - Flat Top window.\n', ...
                                                                          'gausswin \t - Gaussian window.\n', ...
                                                                          'kaiser \t - Kaiser window.\n', ...
                                                                          'nuttallwin \t - Nuttall defined minimum 4-term Blackman-Harris window.\n', ...
                                                                          'parzenwin \t - Parzen (de la Valle-Poussin) window.\n', ...
                                                                          'taylorwin \t - Taylor window.\n', ...
                                                                          'tukeywin \t - Tukey window.\n', ...
                                                                          'triang \t - Triangular window.\n' ...
                                                                        ])},'cat','Modeling Parameters'), ...
    arg({'epochTimeLims','EpochTimeLimits'},[],[],'Sub-epoch time limits (sec). This is relative to event time (e.g. [-1 2]). Default is the full epoch time range','cat','Modeling Parameters'), ...
    arg({'prctWinToSample','WindowSamplePercent'},100,[1 100],'Percent of windows to sample','cat','Modeling Parameters'), ...
    arg_subtoggle({'normalize','NormalizeData'},[],@pre_normData,'Z-normalize data within windows. Note this is not recommended for short windows','cat','Window Preprocessing'), ...
    arg_subtoggle({'detrend','Detrend'},{}, ...
    {arg({'method','DetrendingMethod'},'constant',{'linear','constant'},{'Detrend data within each window.', ...
    sprintf(['\n' ...
    'Linear: removes the least-squares fit of a straight line.\n' ...
    'Constant: removes the mean from each trial (centering)' ...
    ]) ...
    } ...
    )},'Detrend or center each time window','cat','Window Preprocessing'), ...
    arg({'timer','Timer'},false,[],'Activate timer. Times are stored in EEG.CAT.Model.timeelapsed'), ...
    arg({'setArgDirectMode','SetArgDirectMode'},true,[],'Set arg_direct mode to true. Can improve speed when number of windows is large. Disable if calling this function repeatedly in a tight loop.'), ...
    arg({'verb','VerbosityLevel'},verb,{int32(0) int32(1) int32(2)},'Verbosity level. 0 = no output, 1 = text, 2 = graphical') ...
    );



% commit EEG variable to workspace
[data g] = hlp_splitstruct(g,{'EEG'});
arg_toworkspace(data);
clear data;

% do some error-checking
if isempty(g.epochTimeLims)
    g.epochTimeLims = [EEG.xmin EEG.xmax]; end
if ~(all(g.epochTimeLims>=EEG.xmin) && all(g.epochTimeLims<=EEG.xmax))
    error('Epoch time limits must be within the range [%0.3g %0.3g]',EEG.xmin,EEG.xmax); end
if isempty(g.morder) || length(g.morder)>1
    error('invalid entry for field ''morder.'' Make sure the model order is a single integer.'); end

if nargout > 1, cfg = g; end

%% do some adjustments to parameters

% ensure we are using the right model order
g.algorithm.morder = g.morder;

if rem(g.winstep,1/EEG.srate)
    if g.winstep<1/EEG.srate
        g.winstep = 1/EEG.srate;
    else
        % adjust step size to nearest multiple of sampling interval
        g.winstep = g.winstep-rem(g.winstep,1/EEG.srate);
    end
    
    if g.verb,
        fprintf('Adjusting window step size to nearest multiple of sampling interval\n');
        fprintf('\tstep size is now %0.5g sec\n',g.winstep);
    end
end

if rem(g.winlen,1/EEG.srate)
    if g.winlen<1/EEG.srate
        g.winlen = 1/EEG.srate;
    else
        % adjust window length to nearest multiple of sampling interval
        g.winlen = g.winlen-rem(g.winlen,1/EEG.srate);
    end
    
    if g.verb,
        fprintf('Adjusting window length to nearest multiple of sampling interval\n');
        fprintf('\twindow length is now %0.5g sec\n',g.winlen);
    end
end
if g.winlen > EEG.xmax-EEG.xmin
    g.winlen = EEG.xmax-EEG.xmin;
    if g.verb,
        fprintf('Window length exceeds epoch length. Adjusting window length to match epoch length\n');
        fprintf('\twindow length is now %0.5g sec\n',g.winlen);
    end
end
tidx = getindex(EEG.CAT.times,g.epochTimeLims*1000);
if ~all(isequal(EEG.CAT.times(tidx),g.epochTimeLims*1000))
    
    g.epochTimeLims = EEG.CAT.times(tidx)/1000;
    
    if g.verb
        fprintf('Adjusting epoch time limits to match sampling interval\n');
        fprintf('\tepoch limits are now [%0.5g, %0.5g] sec\n',g.epochTimeLims(1),g.epochTimeLims(2));
    end
end

winLenPnts  = round(g.winlen*EEG.srate); % window size in points
winStepPnts = round(g.winstep*EEG.srate);

if isempty(g.winStartIdx)
    % starting point of each window (points)
    g.winStartIdx  = tidx(1):winStepPnts:(tidx(2)-winLenPnts)+1;
    %g.winStartIdx  =  round((double(g.epochTimeLims(1):g.winstep:g.epochTimeLims(2)-g.winlen)*EEG.srate)+1;
end

if g.prctWinToSample<100
    % randomly select percentage of windows to work with
    randwin = randperm(length(g.winStartIdx));
    randwin = sort(randwin(1:ceil(length(g.winStartIdx)*g.prctWinToSample/100)));
    g.winStartIdx = g.winStartIdx(randwin);
end

numWins   = length(g.winStartIdx);

% construct the data taper
g.taper = window_func(g.taperfcn,winLenPnts).';


%% Prepare data for model fitting

% initialize results arrays
[AR PE RC mu th]  = deal(cell(1,numWins));

if g.verb==2
    waitbarTitle = sprintf('Fitting VAR[%d] model (%s)...', ...
                        g.morder, ...
                        num2str(g.algorithm.arg_selection));
    
    multiWaitbar(waitbarTitle,'Reset');
    multiWaitbar(waitbarTitle, ...
                 'Color', hlp_getNextUniqueColor, ...
                 'CanCancel','on', ...
                 'CancelFcn',@(a,b) disp('[Cancel requested. Please wait...]'));
end

if g.normalize.arg_selection
    % normalize each window separately
    if g.verb, fprintf('Normalizing each window across %s...',g.normalize.method{:}); end
    for t=1:numWins
        winpnts = g.winStartIdx(t):g.winStartIdx(t)+winLenPnts-1;
        EEG.CAT.srcdata(:,winpnts,:) = pre_normData('data',EEG.CAT.srcdata(:,winpnts,:),'method',g.normalize.method,'verb',0,'arg_direct',true);
    end
    if g.verb, fprintf('done.\n'); end
end

if g.detrend.arg_selection
    % detrend each window separately
    if g.verb, fprintf('%s detrending each window...\n',firstcaps(g.detrend.method)); end
    for ch=1:EEG.CAT.nbchan
        EEG.CAT.srcdata(ch,:,:) = locdetrend_siftmod(squeeze(EEG.CAT.srcdata(ch,:,:)), ...
            EEG.srate,[g.winlen g.winstep],g.detrend.method);
    end
    if g.verb, fprintf('done.\n'); end
end

if g.timer
    timeElapsed = nan(1,numWins);
else
    timeElapsed = [];
end

%% Main loop: fit MVAR models to each window

if g.setArgDirectMode
    % set the arg_direct flag
    % to improve speed
    g = arg_setdirect(g,true);
end

for t=1:numWins
    
    if g.timer, tic; end
    
    % get indices of all data points (samples) in current window
    winpnts = g.winStartIdx(t):g.winStartIdx(t)+winLenPnts-1;
    
    % execute the model-fitting algorithm
    algFcnName = hlp_microcache('algos',@hlp_getMVARalgorithms,'mfileNameOnly',g.algorithm.arg_selection);
    switch nargout(algFcnName)
        case 2
            [AR{t} PE{t}] = feval(algFcnName, ...
                'data',bsxfun(@times,g.taper,EEG.CAT.srcdata(:,winpnts,:)), ...
                g.algorithm);
        case 3
            [AR{t} PE{t} argsout] = feval(algFcnName, ...
                                   'data',bsxfun(@times,g.taper,EEG.CAT.srcdata(:,winpnts,:)),...
                                    g.algorithm);
            if isstruct(argsout)
                % store contents of argsout fields in cell array at index t
                % e.g. fieldname{t} = argsout.(fieldname)
                fnames = fieldnames(argsout);
                for k=1:length(fnames)
                    eval([fnames{k} '{t}=argsout.(''' fnames{k} ''');']);
                end
            end
        otherwise
            error('SIFT:est_fitMVAR:badAlgArgs','%s must output either 2 or 3 arguments',algFcnName);
    end
        
    if g.verb==2
        drawnow;
        % graphical waitbar
        cancel = multiWaitbar(waitbarTitle,t/numWins);
        if cancel
            if strcmpi('yes',questdlg2( ...
                            'Are you sure you want to cancel?', ...
                            'Model Fitting','Yes','No','No'));
                Model = [];
                multiWaitbar(waitbarTitle,'Close');
                return;
            else
                multiWaitbar(waitbarTitle,'ResetCancel',true);
            end
        end
    end
    
    if g.timer, timeElapsed(t) = toc; end
end

%% Do some cleanup
clear('-regexp','mvar_*')
if g.verb==2
    multiWaitbar(waitbarTitle,'Close'); 
end

%% Construct Model object
Model = hlp_sift_emptymodel;

Model.AR = AR;
Model.PE = PE;
Model.RC = RC;
Model.mu = mu;
Model.th = th;
Model.winStartTimes = (g.winStartIdx-1)/EEG.srate;
Model.morder        = g.morder;
Model.winstep       = g.winstep;
Model.winlen        = g.winlen;
Model.algorithm     = g.algorithm.arg_selection;
Model.modelclass    = 'mvar';
Model.timeelapsed   = timeElapsed;
Model.normalize     = g.normalize;
Model.modelapproach = 'Segmentation VAR';
Model.taperFcn      = g.taperfcn;

switch lower(g.algorithm.arg_selection)
    case 'group lasso dal/scsa'
        %     Model.ww = ww;
        Model.lambda = g.algorithm.dal_args.lambda;
    case 'group lasso (admm)'
        Model.lambda = g.algorithm.admm_args.lambda;
        Model.rho    = g.algorithm.admm_args.rho;
        Model.alpha  = g.algorithm.admm_args.alpha;
    case 'scsa em'
        Model.SCSA = SCSA;
end
